@@ -1,6 +1,18 @@
+/*
+ * Copyright (C) 2009 Canonical Ltd.
+ * Author: Robert Ancell <robert.ancell@canonical.com>
+ * 
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any later
+ * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
+ * license.
+ */
+
 #include <string.h>
 #include <sane/sane.h>
 #include <sane/saneopts.h>
+#include <glib/gi18n.h>
 
 #include "scanner.h"
 
@@ -10,6 +22,7 @@ enum {
     UPDATE_DEVICES,
     GOT_PAGE_INFO,
     GOT_LINE,
+    SCAN_FAILED,
     IMAGE_DONE,
     LAST_SIGNAL
 };
@@ -36,6 +49,8 @@ struct ScannerPrivate
 {
     GAsyncQueue *scan_queue;
     gint dpi;
+    gboolean running;
+    GThread *thread;
 };
 
 G_DEFINE_TYPE (Scanner, scanner, G_TYPE_OBJECT);
@@ -45,7 +60,48 @@ static gboolean
 send_signal (SignalInfo *info)
 {
     g_signal_emit (info->instance, signals[info->sig], 0, info->data);
+    
+    switch (info->sig) {
+    case UPDATE_DEVICES:
+        {
+            GList *iter, *devices = info->data;
+            for (iter = devices; iter; iter = iter->next) {
+                ScanDevice *device = iter->data;
+                g_free (device->name);
+                g_free (device->label);
+                g_free (device);
+            }
+            g_list_free (devices);
+        }
+        break;
+    case GOT_PAGE_INFO:
+        {
+            ScanPageInfo *page_info = info->data;
+            g_free (page_info);
+        }
+        break;
+    case GOT_LINE:
+        {
+            ScanLine *line = info->data;
+            g_free(line->data);
+            g_free(line);
+        }
+        break;
+    case SCAN_FAILED:
+        {
+            GError *error = info->data;
+            g_error_free (error);
+        }
+        break;
+    default:
+    case READY:
+    case IMAGE_DONE:
+    case LAST_SIGNAL:
+        g_assert (info->data == NULL);
+        break;
+    }
     g_free (info);
+
     return FALSE;
 }
 
@@ -121,7 +177,7 @@ scan_thread (Scanner *scanner)
              SANE_VERSION_MINOR(version_code),
              SANE_VERSION_BUILD(version_code));
 
-    while (1) {
+    while (scanner->priv->running) {
         /* Get device to use */
         if (state == STATE_IDLE) {
             GTimeVal timeout = { 1, 0 };
@@ -131,6 +187,10 @@ scan_thread (Scanner *scanner)
                 state = STATE_CLOSE;
         }
         
+        /* Interrupted */
+        if (!scanner->priv->running)
+            break;
+        
         switch (state) {
         case STATE_IDLE:
             if (device == NULL) {
@@ -138,8 +198,11 @@ scan_thread (Scanner *scanner)
             } else if (device[0] != '\0') {
                 status = sane_open (device, &handle);
                 if (status != SANE_STATUS_GOOD) {
-                    g_warning ("Unable to get open device: Error %s", sane_strstatus(status));
-                    handle = NULL;
+                    g_warning ("Unable to get open device: %s", sane_strstatus (status));
+                    emit_signal (scanner, SCAN_FAILED,
+                                 g_error_new (SCANNER_TYPE, status,
+                                              /* Error displayed when cannot connect to scanner */
+                                              _("Unable to connect to scanner")));
                     state = STATE_CLOSE;
                 }
                 else {
@@ -148,7 +211,7 @@ scan_thread (Scanner *scanner)
                 }
             }
             break;
-            
+
         case STATE_GET_OPTION:
             option = sane_get_option_descriptor (handle, option_index);
             if (!option) {
@@ -166,6 +229,12 @@ scan_thread (Scanner *scanner)
                 }
                 if (option->name && strcmp (option->name, SANE_NAME_SCAN_RESOLUTION) == 0) {
                     SANE_Int dpi = scanner->priv->dpi;
+                    if (option->constraint_type == SANE_CONSTRAINT_RANGE) {
+                        if (dpi < option->constraint.range->min)
+                            dpi = option->constraint.range->min;
+                        if (dpi > option->constraint.range->max)
+                            dpi = option->constraint.range->max;
+                    }
                     sane_control_option (handle, option_index, SANE_ACTION_SET_VALUE, &dpi, NULL);
                 }
                 option_index++;
@@ -175,7 +244,11 @@ scan_thread (Scanner *scanner)
         case STATE_START:
             status = sane_start (handle);
             if (status != SANE_STATUS_GOOD) {
-                g_warning ("Unable to start device: Error %s", sane_strstatus(status));
+                g_warning ("Unable to start device: Error %s", sane_strstatus (status));
+                emit_signal (scanner, SCAN_FAILED,
+                             g_error_new (SCANNER_TYPE, status,
+                                          /* Error display when unable to start scan */
+                                          _("Unable to start scan")));
                 state = STATE_CLOSE;
             } else {
                 state = STATE_GET_PARAMETERS;
@@ -185,7 +258,11 @@ scan_thread (Scanner *scanner)
         case STATE_GET_PARAMETERS:
             status = sane_get_parameters (handle, &parameters);
             if (status != SANE_STATUS_GOOD) {
-                g_warning ("Unable to get device parameters: Error %s", sane_strstatus(status));
+                g_warning ("Unable to get device parameters: Error %s", sane_strstatus (status));
+                emit_signal (scanner, SCAN_FAILED,
+                             g_error_new (SCANNER_TYPE, status,
+                                          /* Error displayed when communication with scanner broken */
+                                          _("Error communicating with scanner")));
                 state = STATE_CLOSE;
             } else {
                 ScanPageInfo *info;
@@ -209,7 +286,11 @@ scan_thread (Scanner *scanner)
                 bytes_remaining == parameters.bytes_per_line) {
                 state = STATE_CLOSE;
             } else if (status != SANE_STATUS_GOOD) {
-                g_warning ("Unable to read frame from device: Error %s", sane_strstatus(status));
+                g_warning ("Unable to read frame from device: Error %s", sane_strstatus (status));
+                emit_signal (scanner, SCAN_FAILED,
+                             g_error_new (SCANNER_TYPE, status,
+                                          /* Error displayed when communication with scanner broken */
+                                          _("Error communicating with scanner")));
                 state = STATE_CLOSE;
             } else {
                 bytes_remaining -= n_read;
@@ -272,14 +353,6 @@ scan_thread (Scanner *scanner)
 }
 
 
-/* FIXME: Do I need to call this anytime?
-static void
-scanner_close ()
-{
-    sane_exit ();
-}*/
-
-
 Scanner *
 scanner_new ()
 {
@@ -302,6 +375,22 @@ scanner_cancel (Scanner *scanner)
 }
 
 
+void scanner_free (Scanner *scanner)
+{
+    g_debug ("Stopping scan thread");
+    scanner->priv->running = FALSE;
+    g_async_queue_push (scanner->priv->scan_queue, "");
+    g_thread_join (scanner->priv->thread);
+
+    g_async_queue_unref (scanner->priv->scan_queue);
+    g_object_unref (scanner);
+
+    g_debug ("Closing SANE interface");
+    sane_exit ();
+    g_debug ("SANE interface closed");
+}
+
+
 static void
 scanner_class_init (ScannerClass *klass)
 {
@@ -337,6 +426,14 @@ scanner_class_init (ScannerClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__POINTER,
                       G_TYPE_NONE, 1, G_TYPE_POINTER);
+    signals[SCAN_FAILED] =
+        g_signal_new ("scan-failed",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ScannerClass, scan_failed),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__POINTER,
+                      G_TYPE_NONE, 1, G_TYPE_POINTER);
     signals[IMAGE_DONE] =
         g_signal_new ("image-done",
                       G_TYPE_FROM_CLASS (klass),
@@ -356,9 +453,10 @@ scanner_init (Scanner *scanner)
     GError *error = NULL;
     
     scanner->priv = G_TYPE_INSTANCE_GET_PRIVATE (scanner, SCANNER_TYPE, ScannerPrivate);
-    
+
+    scanner->priv->running = TRUE;
     scanner->priv->scan_queue = g_async_queue_new ();
-    g_thread_create ((GThreadFunc) scan_thread, scanner, FALSE, &error);
+    scanner->priv->thread = g_thread_create ((GThreadFunc) scan_thread, scanner, TRUE, &error);
     if (error) {
         g_critical ("Unable to create thread: %s", error->message);
         g_error_free (error);
