@@ -1,3 +1,14 @@
+/*
+ * Copyright (C) 2009 Canonical Ltd.
+ * Author: Robert Ancell <robert.ancell@canonical.com>
+ * 
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any later
+ * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
+ * license.
+ */
+
 #ifndef _SCANNER_H_
 #define _SCANNER_H_
 
@@ -5,9 +16,9 @@
 
 G_BEGIN_DECLS
 
-#define SCANNER_TYPE                (scanner_get_type ())
-#define SCANNER(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCANNER_TYPE, Scanner))
-    
+#define SCANNER_TYPE  (scanner_get_type ())
+#define SCANNER(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCANNER_TYPE, Scanner))
+
 
 typedef struct
 {
@@ -57,6 +68,7 @@ typedef struct
     void (*update_devices) (Scanner *scanner, GList *devices);
     void (*got_page_info) (Scanner *scanner, ScanPageInfo *info);
     void (*got_line) (Scanner *scanner, ScanLine *line);
+    void (*scan_failed) (Scanner *scanner, GError *error);
     void (*image_done) (Scanner *scanner);
 } ScannerClass;
 
@@ -67,4 +79,6 @@ void scanner_scan (Scanner *scanner, const char *device, gint dpi);
 
 void scanner_cancel (Scanner *scanner);
 
+void scanner_free (Scanner *scanner);
+
 #endif /* _SCANNER_H_ */
